package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.StrategyAndMetricsEnum;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.opt.bo.CrossSchemeListBO;
import net.wanji.opt.bo.ProblemSchemeBO;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.po.strategy.SceneStrategyPO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.service.SchemeEvaluateService;
import net.wanji.opt.vo.RunningEvaluateMetricsDetailVO;
import net.wanji.opt.vo.SchemeEvaluateCrossSchemeListVO;
import net.wanji.opt.vo.SchemeEvaluateProblemSchemeVO;
import net.wanji.opt.vo.SchemeEvaluateStrategyMetricMenuVO;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/8/25 16:09
 */
@Slf4j
@Service
public class SchemeEvaluateServiceImpl implements SchemeEvaluateService {
    private final CrossDataHistMapper crossDataHistMapper;
    private final RunningEvaluateServiceImpl runningEvaluateService;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final SceneMapper sceneMapper;
    private final SceneStrategyMapper sceneStrategyMapper;
    private final StrategyMapper strategyMapper;
    private final BaseCrossSchedulesMapper baseCrossSchedulesMapper;
    private final BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper;

    public SchemeEvaluateServiceImpl(CrossDataHistMapper crossDataHistMapper,
                                     RunningEvaluateServiceImpl runningEvaluateService,
                                     @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper,
                                     @Qualifier("baseCrossSchemeMapper") BaseCrossSchemeMapper baseCrossSchemeMapper,
                                     @Qualifier("sceneMapper") SceneMapper sceneMapper,
                                     @Qualifier("sceneStrategyMapper") SceneStrategyMapper sceneStrategyMapper,
                                     @Qualifier("strategyMapper") StrategyMapper strategyMapper,
                                     @Qualifier("baseCrossSchedulesMapper") BaseCrossSchedulesMapper baseCrossSchedulesMapper,
                                     @Qualifier("baseCrossSchedulesPlanMapper") BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper) {
        this.crossDataHistMapper = crossDataHistMapper;
        this.runningEvaluateService = runningEvaluateService;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.sceneMapper = sceneMapper;
        this.sceneStrategyMapper = sceneStrategyMapper;
        this.strategyMapper = strategyMapper;
        this.baseCrossSchedulesMapper = baseCrossSchedulesMapper;
        this.baseCrossSchedulesPlanMapper = baseCrossSchedulesPlanMapper;
    }

    @Override
    public SchemeEvaluateProblemSchemeVO problemSchemeList(ProblemSchemeBO problemSchemeBO) {
        SchemeEvaluateProblemSchemeVO res = new SchemeEvaluateProblemSchemeVO();
        String crossId = problemSchemeBO.getCrossId();
        Date startDate = problemSchemeBO.getStartDate();
        Date endDate = problemSchemeBO.getEndDate();
        endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        int startDateStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endDateStamp = (int) (endDate.getTime() / 1000);
        String startTime = problemSchemeBO.getStartTime();
        String endTime = problemSchemeBO.getEndTime();
        List<CrossDataHistPO> crossDataHistPOList =
                crossDataHistMapper.selectByCrossIdAndStartEnd(crossId, startDateStamp, endDateStamp);
        // 构造时间-问题结果集
        Map<String, List<CrossDataHistPO>> timeProblemMap = getTimeProblemMap(
                crossDataHistPOList, endTime, startTime);
        // 构造结果
        int problemCounts = 0; // 问题次数
        if (CollectionUtil.isNotEmpty(timeProblemMap)) {
            for (List<CrossDataHistPO> list : timeProblemMap.values()) {
                problemCounts += list.size();
            }
        }
        res.setProblemCounts(problemCounts); // 问题方案数量
        // 构造问题方案列表（根据日期）
        Map<Date, List<CrossDataHistPO>> dateProblemsMap = buildDateProblemsMap(timeProblemMap); // 构造日期-问题结果集
        Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap =
                buildDateSchemesMap(dateProblemsMap, crossId); // 构造日期-问题方案结果集

        List<SchemeEvaluateProblemSchemeVO.ByDateVO> byDateVOList = buildProblemSchemeListByDate(
                dateProblemsMap, dateSchemesMap);
        res.setProblemSchemeListByDate(byDateVOList);
        // 构造计数
        int schemeCounts = calcSchemeCounts(dateSchemesMap);
        res.setSchemeCounts(schemeCounts);
        res.setEfficiencyProblemCounts(calcProblemCounts(
                byDateVOList, SchemeEvaluateProblemSchemeVO.ByDateVO::getEfficiencyProblemCounts));
        res.setBalanceProblemCounts(calcProblemCounts(
                byDateVOList, SchemeEvaluateProblemSchemeVO.ByDateVO::getBalanceProblemCounts));
        res.setSecurityProblemCounts(calcProblemCounts(
                byDateVOList, SchemeEvaluateProblemSchemeVO.ByDateVO::getSecurityProblemCounts));
        // 构造问题方案列表（根据方案）
        res.setProblemSchemeListByScheme(buildProblemSchemeListByScheme(dateSchemesMap));

        return res;
    }

    @Override
    public List<SchemeEvaluateStrategyMetricMenuVO> strategyMetricMenu(CrossIdBO crossIdBO) {
        List<SchemeEvaluateStrategyMetricMenuVO> voList = new ArrayList<>();

        for (Map.Entry<StrategyAndMetricsEnum.Strategy, List<StrategyAndMetricsEnum.Metrics>> entry
                : StrategyAndMetricsEnum.STRATEGY_METRICS_MAP.entrySet()) {
            SchemeEvaluateStrategyMetricMenuVO vo = new SchemeEvaluateStrategyMetricMenuVO();
            vo.setStrategyCode(entry.getKey().getCode());
            vo.setStrategyName(entry.getKey().getMsg());
            List<SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric> metricInfoList = buildEvaluateMetrics(entry);
            vo.setMetricList(metricInfoList);
            voList.add(vo);
        }
        return voList;
    }

    private List<SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric> buildEvaluateMetrics(
            Map.Entry<StrategyAndMetricsEnum.Strategy, List<StrategyAndMetricsEnum.Metrics>> entry) {
        List<SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric> metricInfoList = new ArrayList<>();
        for (StrategyAndMetricsEnum.Metrics metric : entry.getValue()) {
            SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric evaluateMetric =
                    new SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric();
            evaluateMetric.setMetricCode(metric.getCode());
            evaluateMetric.setMetricName(metric.getDescription());
            metricInfoList.add(evaluateMetric);
        }
        return metricInfoList;
    }

    @Override
    public List<SchemeEvaluateCrossSchemeListVO> crossSchemeList(CrossSchemeListBO crossSchemeListBO) {
        String crossId = crossSchemeListBO.getCrossId();
        String schemeNoOrName = crossSchemeListBO.getSchemeNoOrName();
        String strategyCode = crossSchemeListBO.getStrategyCode();
        // 查找方案
        List<BaseCrossSchemePO> baseCrossSchemePOList = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(schemeNoOrName)) {
            if (isNumeric(schemeNoOrName)) { // 方案号
                String schemeNo = schemeNoOrName;
                BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectByCrossIdAndSchemeNo(
                        crossId, Integer.parseInt(schemeNo));
                baseCrossSchemePOList.add(baseCrossSchemePO);
            } else { // 方案名模糊搜索
                String schemeName = schemeNoOrName;
                baseCrossSchemePOList = baseCrossSchemeMapper.selectByCrossIdAndPartialSchemeName(
                        crossId, schemeName);
            }
        } else { // 全部方案
            baseCrossSchemePOList = baseCrossSchemeMapper.selectByCrossId(crossId);
        }

        Integer weekDay = crossSchemeListBO.getWeekDay();
        // 找出当前路口已执行的调度，在某天对应的计划
        BaseCrossSchedulesPO baseCrossSchedulesPO = baseCrossSchedulesMapper.selectExecByCrossId(crossId);
        if (ObjectUtil.isEmpty(baseCrossSchedulesPO)) {
            throw new RuntimeException("该路口没有正在执行的调度");
        }
        Integer scheduleId = baseCrossSchedulesPO.getId();
        List<BaseCrossSchedulesPlanPO> baseCrossSchedulesPlanPOList =
                baseCrossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, scheduleId);
        List<BaseCrossSchedulesPlanPO> filteredList = baseCrossSchedulesPlanPOList.stream()
                .filter(po -> po.getWeek().equals(weekDay))
                .collect(Collectors.toList());

        List<SchemeEvaluateCrossSchemeListVO> res = new ArrayList<>();
        // 根据计划 ID 和方案 ID，获取时段信息
        for (BaseCrossSchedulesPlanPO baseCrossSchedulesPlanPO : filteredList) {
            Integer planId = baseCrossSchedulesPlanPO.getPlanId();
            for (BaseCrossSchemePO baseCrossSchemePO : baseCrossSchemePOList) {
                Integer schemeId = baseCrossSchemePO.getId();
                List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectByCrossIdPlanIdAndSchemeId(
                        crossId, planId, schemeId);
                // todo 按控制模式和策略入参筛选结果
                for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                    SchemeEvaluateCrossSchemeListVO vo =
                            buildSchemeEvaluateCrossSchemeListVO(baseCrossSchemePO, crossSectionPO);
                    res.add(vo);
                }
            }
        }
        // 特殊日期根据方案 ID 筛选重复方案
        if (weekDay == 0) {
            List<SchemeEvaluateCrossSchemeListVO> sortedUniqueList = res.stream()
                    .collect(Collectors.toMap(
                            SchemeEvaluateCrossSchemeListVO::getSchemeId,
                            vo -> vo,
                            (existing, replacement) -> existing))
                    .values()
                    .stream()
                    .sorted(Comparator.comparing(SchemeEvaluateCrossSchemeListVO::getStartTime))
                    .collect(Collectors.toList());
            return sortedUniqueList;
        }
        // 按照 startTime 排序
        List<SchemeEvaluateCrossSchemeListVO> sortedList = res.stream()
                .sorted(Comparator.comparing(SchemeEvaluateCrossSchemeListVO::getStartTime))
                .collect(Collectors.toList());
        return sortedList;
    }

    @NotNull
    private static SchemeEvaluateCrossSchemeListVO buildSchemeEvaluateCrossSchemeListVO
            (BaseCrossSchemePO baseCrossSchemePO, CrossSectionPO crossSectionPO) {
        SchemeEvaluateCrossSchemeListVO vo = new SchemeEvaluateCrossSchemeListVO();
        vo.setSchemeName(baseCrossSchemePO.getName());
        vo.setSchemeId(baseCrossSchemePO.getId());
        vo.setStartTime(crossSectionPO.getStartTime());
        vo.setEndTime(crossSectionPO.getEndTime());
        vo.setSectionId(crossSectionPO.getId());
        vo.setSchemeNo(baseCrossSchemePO.getSchemeNo());
        vo.setCycle(baseCrossSchemePO.getCycle());
        vo.setNote("");
        return vo;
    }

    private boolean isNumeric(String str) {
        return str != null && str.matches("[0-9]+");
    }


    private List<SchemeEvaluateProblemSchemeVO.BySchemeVO> buildProblemSchemeListByScheme(
            Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap) {
        // 使用一个Map来临时存储按方案名称分组的数据
        Map<String, List<SchemeEvaluateProblemSchemeVO.SchemeDatetime>> tempMap = new HashMap<>();

        for (Map.Entry<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> entry : dateSchemesMap.entrySet()) {
            Date date = entry.getKey();
            List<RunningEvaluateMetricsDetailVO.ProblemScheme> problemSchemes = entry.getValue();

            for (RunningEvaluateMetricsDetailVO.ProblemScheme problemScheme : problemSchemes) {
                Integer schemeId = problemScheme.getSchemeId();
                String schemeName = problemScheme.getSchemeName();
                String schemeIdName = schemeId + "_" + schemeName;
                SchemeEvaluateProblemSchemeVO.SchemeDatetime schemeDatetime =
                        new SchemeEvaluateProblemSchemeVO.SchemeDatetime();
                schemeDatetime.setYear(date);
                schemeDatetime.setWeek(DateUtil.dayOfWeek(date));
                schemeDatetime.setProblemDate(date);
                schemeDatetime.setStartTime(problemScheme.getStartTime());
                schemeDatetime.setEndTime(problemScheme.getEndTime());

                tempMap.computeIfAbsent(schemeIdName, k -> new ArrayList<>()).add(schemeDatetime);
            }
        }
        // 构建最终的 List<BySchemeVO>
        List<SchemeEvaluateProblemSchemeVO.BySchemeVO> bySchemeVOList = tempMap.entrySet().stream()
                .map(entry -> {
                    SchemeEvaluateProblemSchemeVO.BySchemeVO bySchemeVO = new SchemeEvaluateProblemSchemeVO.BySchemeVO();
                    String schemeIdName = entry.getKey();
                    String[] s = schemeIdName.split("_");
                    bySchemeVO.setSchemeId(Integer.parseInt(s[0]));
                    bySchemeVO.setSchemeName(s[1]);
                    bySchemeVO.setSchemeDatetimeList(entry.getValue());
                    return bySchemeVO;
                })
                .collect(Collectors.toList());
        return bySchemeVOList;
    }

    private int calcProblemCounts(List<SchemeEvaluateProblemSchemeVO.ByDateVO> byDateVOList,
                                  Function<SchemeEvaluateProblemSchemeVO.ByDateVO, Integer> getCount) {
        return byDateVOList.stream().mapToInt(getCount::apply).sum();
    }

    private int calcSchemeCounts(Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap) {
        int schemeCounts = 0;
        for (List<RunningEvaluateMetricsDetailVO.ProblemScheme> list : dateSchemesMap.values()) {
            schemeCounts += list.size();
        }
        return schemeCounts;
    }

    @NotNull
    private List<SchemeEvaluateProblemSchemeVO.ByDateVO> buildProblemSchemeListByDate(
            Map<Date, List<CrossDataHistPO>> dateProblemsMap,
            Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap) {
        List<SchemeEvaluateProblemSchemeVO.ByDateVO> byDateVOList = new ArrayList<>();

        for (Map.Entry<Date, List<CrossDataHistPO>> entry : dateProblemsMap.entrySet()) {
            Date key = entry.getKey();
            SchemeEvaluateProblemSchemeVO.ByDateVO byDateVO = new SchemeEvaluateProblemSchemeVO.ByDateVO();
            byDateVO.setProblemDate(key);
            List<RunningEvaluateMetricsDetailVO.ProblemScheme> problemSchemeList = dateSchemesMap.get(key);
            byDateVO.setSectionSechemeList(problemSchemeList);
            List<CrossDataHistPO> value = entry.getValue();

            int efficiencyProblemCounts = 0;
            int balanceProblemCounts = 0;
            int securityProblemCounts = 0;
            for (CrossDataHistPO crossDataHistPO : value) {
                Integer status = crossDataHistPO.getStatus();
                // 根据场景查询策略
                ScenePO scenePO = sceneMapper.selectBySceneNum(status);
                Integer sceneId = scenePO.getId();
                List<SceneStrategyPO> sceneStrategyList = sceneStrategyMapper.selectBySceneId(sceneId);
                for (SceneStrategyPO sceneStrategyPO : sceneStrategyList) {
                    Integer strategyId = sceneStrategyPO.getStrategyId();
                    StrategyPO strategyPO = strategyMapper.selectById(strategyId);
                    String strategyCode = strategyPO.getStrategyCode();
                    if (strategyCode.equals(StrategyAndMetricsEnum.Strategy.EFFICIENCY.getCode())) {
                        efficiencyProblemCounts += 1;
                    } else if (strategyCode.equals(StrategyAndMetricsEnum.Strategy.BALANCE.getCode())) {
                        balanceProblemCounts += 1;
                    } else if (strategyCode.equals(StrategyAndMetricsEnum.Strategy.SECURITY.getCode())) {
                        securityProblemCounts += 1;
                    }
                }

            }
            byDateVO.setEfficiencyProblemCounts(efficiencyProblemCounts);
            byDateVO.setBalanceProblemCounts(balanceProblemCounts);
            byDateVO.setSecurityProblemCounts(securityProblemCounts);
            byDateVOList.add(byDateVO);
        }
        return byDateVOList;
    }

    private Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> buildDateSchemesMap(
            Map<Date, List<CrossDataHistPO>> dateProblemsMap, String crossId) {

        Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap = new HashMap<>();
        // 获取当前计划ID
        for (Map.Entry<Date, List<CrossDataHistPO>> dateListEntry : dateProblemsMap.entrySet()) {
            List<RunningEvaluateMetricsDetailVO.ProblemScheme> problemSchemeList = new ArrayList<>();
            Date datetime = dateListEntry.getKey();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String dateStr = sdf.format(datetime);
            Integer planId = runningEvaluateService.findPlanId(datetime, dateStr, crossId);
            List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectByCrossAndPlan(crossId, planId);
            for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                String sectionStartTime = crossSectionPO.getStartTime();
                String sectionEndTime = crossSectionPO.getEndTime();
                List<CrossDataHistPO> value = dateListEntry.getValue();
                for (CrossDataHistPO crossDataHistPO : value) {
                    SimpleDateFormat sdf2 = new SimpleDateFormat("HH:mm");
                    Date abnormalEventStartTime = crossDataHistPO.getStartTime();
                    Integer duration = crossDataHistPO.getDuration();
                    Date abnormalEventEndTime = DateUtil.offsetMinute(abnormalEventStartTime, duration);
                    String eventStartTime = sdf2.format(abnormalEventStartTime);
                    String eventEndTime = sdf2.format(abnormalEventEndTime);
                    // 比较两组时间段是否相交
                    boolean isIntersecting = runningEvaluateService.isTimeIntersecting(
                            eventStartTime, eventEndTime, sectionStartTime, sectionEndTime);
                    runningEvaluateService.buildResIfIntersecting(problemSchemeList, crossSectionPO, sectionStartTime,
                            sectionEndTime, isIntersecting, baseCrossSchemeMapper);
                }
            }
            // 去除相同方案
            List<RunningEvaluateMetricsDetailVO.ProblemScheme> uniqueRes = problemSchemeList.stream()
                    .distinct()
                    .collect(Collectors.toList());
            dateSchemesMap.put(datetime, uniqueRes);
        }
        return dateSchemesMap;
    }

    private Map<Date, List<CrossDataHistPO>> buildDateProblemsMap(Map<String, List<CrossDataHistPO>> timeProblemMap) {
        // 创建一个新的Map，用于存储以Date为Key，List<CrossDataHistPO>为Value的数据
        Map<Date, List<CrossDataHistPO>> dateMap = new HashMap<>();
        // 创建一个SimpleDateFormat对象，用于将10位时间戳转换为Date对象
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        // 遍历timeProblemMap的所有entries
        for (List<CrossDataHistPO> list : timeProblemMap.values()) {
            for (CrossDataHistPO po : list) {
                long timestamp = po.getBatchTime() * 1000L; // 将10位时间戳转换为13位
                Date date = new Date(timestamp);
                String formattedDate = sdf.format(date); // 格式化日期为"yyyy-MM-dd"
                try {
                    Date keyDate = sdf.parse(formattedDate); // 将格式化后的日期字符串转回为Date对象
                    // 将CrossDataHistPO对象添加到相应日期的List中
                    dateMap.computeIfAbsent(keyDate, k -> new ArrayList<>()).add(po);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return dateMap;
    }

    @NotNull
    private static Map<String, List<CrossDataHistPO>> getTimeProblemMap(
            List<CrossDataHistPO> crossDataHistPOList, String endTime, String startTime) {
        // 过滤有问题的记录
        List<CrossDataHistPO> filteredList = crossDataHistPOList.stream()
                .filter(po -> po.getStatus() != null && po.getStatus() != 0)
                .collect(Collectors.toList());
        // 保留 duration 最大的一条
        List<CrossDataHistPO> result = filteredList.stream()
                .collect(Collectors.groupingBy(CrossDataHistPO::getStartTime))
                .values().stream()
                .map(list -> list.stream().max(Comparator.comparingInt(CrossDataHistPO::getDuration)).orElse(null))
                .collect(Collectors.toList());
        // 按照 startTime 和 endTime 的 "HH:mm" 的时间区间分组
        Map<String, List<CrossDataHistPO>> groupedByTimeInterval = result.stream()
                .collect(Collectors.groupingBy(po -> {
                    Date startTime1 = po.getStartTime();
                    Date endTime1 = new Date(startTime1.getTime() + po.getDuration() * 60 * 1000);
                    SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
                    return sdf.format(startTime1) + " - " + sdf.format(endTime1);
                }));
        // 过滤 groupedByTimeInterval，保留与 startTime 和 endTime 有交叉的条目
        Map<String, List<CrossDataHistPO>> filteredGroupedByTimeInterval = groupedByTimeInterval.entrySet().stream()
                .filter(entry -> {
                    String[] times = entry.getKey().split(" - ");
                    String groupStartTime = times[0];
                    String groupEndTime = times[1];
                    return (groupStartTime.compareTo(endTime) < 0 || groupStartTime.equals(endTime))
                            && (groupEndTime.compareTo(startTime) > 0 || groupEndTime.equals(startTime));
                })
                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
        return filteredGroupedByTimeInterval;
    }
}
